﻿#include "dialogconfiguration.h"
#include "ui_dialogconfiguration.h"

#include "spinboxunit.h"
#include "settings.h"
#include "language.h"

#include <QDebug>
#include <QLineEdit>
#include <QTranslator>
#include <QFileDialog>
#include <QDir>
#include <QFileInfoList>
#include <QFileInfo>

DialogConfiguration::DialogConfiguration(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::DialogConfiguration)
{
    ui->setupUi(this);
    m_parent=parent;
    Language::setLanguageComboBox(ui->comboBoxInterfaceLanguage);
    connect(ui->pushButtonClose,SIGNAL(clicked(bool)),this,SLOT(close()));
    connect(ui->pushButtonSave,SIGNAL(clicked(bool)),this,SLOT(Save()));
    connect(ui->toolButtonPDFReaderName,SIGNAL(clicked(bool)),this,SLOT(SetPDFReader()));
    //connect(ui->checkBoxInternalPDFReader,SIGNAL(stateChanged(int)),this,SLOT(InternalPDFStatus(int)));
    connect(ui->pushButtonSaveFormat,SIGNAL(clicked(bool)),this,SLOT(SaveFormat()));
    connect(ui->pushButtonSaveDefaultFormat,SIGNAL(clicked(bool)),this,SLOT(SaveDefaultFormat()));
    connect(ui->pushButtonLoadFormat,SIGNAL(clicked(bool)),this,SLOT(LoadFormat()));
    ui->checkBoxInternalPDFReader->setDisabled(true);
    Connect();
    InitSettings();
    ui->tabWidget->setCurrentIndex(0);
    ui->widgetMemory->setOffSetAsDefault();
    ui->lyricsConfig->setOffButtonSetAsDefault();
    ui->textConfig->setOffButtonSetAsDefault();
    ui->chordConfig->setOffButtonSetAsDefault();
    LoadDefaultFormat();
}

void DialogConfiguration::setTranslator(QTranslator *translator)
{
    m_translator=translator;
}

void DialogConfiguration::SaveDefaultFormat()
{
    QSettings s;
    QString defaultfile=ConfigPath()+"/default.fec";
    QSettings fs(defaultfile,QSettings::IniFormat);
    for ( auto children :   ui->scrollArea->findChildren<QLineEdit*>())
    {
     QLineEdit *le=qobject_cast<QLineEdit*>(children);
     QString variable=le->objectName();
     variable.remove(QRegExp("^lineEdit"));
     fs.setValue(variable,le->text());
     }
}

void DialogConfiguration::LoadDefaultFormat()
{
    QSettings s;
    QString defaultfile=ConfigPath()+"/default.fec";
    if ( ! QFile(defaultfile).exists())
    {
       QSettings fs(defaultfile,QSettings::IniFormat);
       fs.setValue("cover_title","<H1>$cover_title</H1>");
       fs.setValue("cover_subtitle","<H2>$cover_subtitle:</H2>");
       fs.setValue("title","<H3>$title</H3>");
       fs.setValue("subtitle","<H4>$subtitle</H1>");
       fs.sync();
    }
   QSettings fs(defaultfile,QSettings::IniFormat);
   for ( auto children :   ui->scrollArea->findChildren<QLineEdit*>())
    {
     QLineEdit *le=qobject_cast<QLineEdit*>(children);
     QString variable=le->objectName();
     variable.remove(QRegExp("^lineEdit"));
     le->setText(fs.value(variable).toString());
     }

}

void DialogConfiguration::LoadFormat()
{
    QSettings s;
    QString filter=tr("Format Export Chord (*.fec)");
    QString fname=QFileDialog::getOpenFileName(this,tr("Choose file to get format"),s.value("LastOpenedDirectory",
                                      QDir::homePath()).toString(),   filter, &filter);
    QSettings fs(fname,QSettings::IniFormat);
   for ( auto children :   ui->scrollArea->findChildren<QLineEdit*>())
    {
     QLineEdit *le=qobject_cast<QLineEdit*>(children);
     QString variable=le->objectName();
     variable.remove(QRegExp("^lineEdit"));
     le->setText(fs.value(variable).toString());
     }

}

void DialogConfiguration::SaveFormat()
{

    QSettings s;
    QString filter=tr("Format Export Chord (*.fec)");

    QString fname=QFileDialog::getSaveFileName(this,tr("Choose file to save format"),s.value("LastOpenedDirectory",
                                      QDir::homePath()).toString(),filter,&filter   );
    if ( ! fname.endsWith(".fec")) fname=fname+".fec";
    QSettings fs(fname,QSettings::IniFormat);
    for ( auto children :   ui->scrollArea->findChildren<QLineEdit*>())
    {
     QLineEdit *le=qobject_cast<QLineEdit*>(children);
     QString variable=le->objectName();
     variable.remove(QRegExp("^lineEdit"));
     fs.setValue(variable,le->text());
     }
}


void DialogConfiguration::SetPDFReader()
{
    QString pdfreader=QFileDialog::getOpenFileName(this,tr("Choose PDF Reader program"),"/");
    ui->lineEditPDFReaderName->setText(pdfreader);
    emit PdfReaderChanged();
}

void DialogConfiguration::Connect()
{
    connect(ui->comboBoxInterfaceLanguage,SIGNAL(currentTextChanged(QString)),this,SLOT(SelectLanguage(QString)));

}

DialogConfiguration::~DialogConfiguration()
{
    delete ui;
}

void DialogConfiguration::SelectLanguage(QString  lang)
{
    Settings s;
    s.setValue("InterfaceLanguage",lang);
    QTranslator tr;
    QString filename=Language::getTranslationQmFileName(lang);
    tr.load(filename);
    if ( lang != "English" )
       {
          qApp->installTranslator(&tr);
          m_translator=&tr;
       }
    else
       {
          qApp->removeTranslator(m_translator);
          m_translator=nullptr;
       }
    Retranslate(lang);
    emit LanguageChanged(lang);
}


void DialogConfiguration::Retranslate(QString)
{
    ui->comboBoxInterfaceLanguage->disconnect();
    ui->retranslateUi(this);
    ui->widgetMemory->Retranslate();
    ui->chordConfig->Retranslate();
    ui->lyricsConfig->Retranslate();
    ui->textConfig->Retranslate();
    //ui->comboBoxInterfaceLanguage->setCurrent;
    connect(ui->comboBoxInterfaceLanguage,SIGNAL(currentTextChanged(QString)),this,SLOT(SelectLanguage(QString)));

}

void DialogConfiguration::InitSettings()
{
    Settings s;
    ui->comboBoxInterfaceLanguage->setCurrentText(s.value("InterfaceLanguage","en").toString());
    ui->comboBoxUnit->setCurrentText(s.value("Unit","mm").toString());
    ui->lineEditArtistName->setText(s.value("ArtistName","").toString());
    ui->lineEditLyricistName->setText(s.value("LyricistName","").toString());
    ui->lineEditArrangerName->setText(s.value("ArrangerName","").toString());
    ui->lineEditComposerName->setText(s.value("ComposerName","").toString());
    ui->lineEditPDFReaderName->setText(s.value("PDFReader","").toString());
    ui->checkBoxOpenLastFileOnStart->setChecked(s.value("OpenLastFileOnStart",Qt::Unchecked).toInt());
    //ui->checkBoxInternalPDFReader->setChecked(s.value("InternalPDFReader",Qt::Checked).toInt());
    //InternalPDFStatus(s.value("InternalPDFReader",Qt::Checked).toInt());
}

QString DialogConfiguration::ConfigPath()
{
    QSettings s;
    QFileInfo fi(s.fileName());
    return fi.absolutePath();
}

void DialogConfiguration::Save()
{
   QSettings s;
   s.setValue("InterfaceLanguage",ui->comboBoxInterfaceLanguage->currentText());
   s.setValue("Unit",ui->comboBoxUnit->currentText());
   s.setValue("ArtistName",ui->lineEditArtistName->text());
   s.setValue("LyricistName",ui->lineEditLyricistName->text());
   s.setValue("ComposerName",ui->lineEditComposerName->text());
   s.setValue("ArrangerName",ui->lineEditArrangerName->text());
   s.setValue("PDFReader",ui->lineEditPDFReaderName->text());
   s.setValue("OpenLastFileOnStart",ui->checkBoxOpenLastFileOnStart->isChecked());
  // s.setValue("InternalPDFReader",ui->checkBoxInternalPDFReader->checkState());

   ui->chordConfig->Save(s.fileName(),"Chord");
   ui->textConfig->Save(s.fileName(),"Text");
   ui->lyricsConfig->Save(s.fileName(),"Lyrics");
   ui->widgetMemory->Save();

   close();
}

void DialogConfiguration::InternalPDFStatus(int status)
{
  ui->lineEditPDFReaderName->setEnabled(status!=Qt::Checked);
  ui->toolButtonPDFReaderName->setEnabled(status!=Qt::Checked);
  ui->labelPDFReaderName->setEnabled(status!=Qt::Checked);
}
